@@ -73,7 +73,9 @@ private:
     template<bool IncludeBoundary = false, class BoundingBoxType, class It, class = IteratorOnly<It>>
     static void construct(BoundingBoxType &out, It from, It to)
     {
-        if (from != to) {
+        if (from == to) {
+            out.defined = false;
+        } else {
             auto it = from;
             out.min = it->template cast<typename PointType::Scalar>();
             out.max = out.min;
