@@ -1,9 +1,16 @@
+///|/ Copyright (c) Prusa Research 2020 - 2023 Vojtěch Bubník @bubnikv, Lukáš Hejl @hejllukas, Lukáš Matěna @lukasmatena, Roman Beránek @zavorka
+///|/
+///|/ PrusaSlicer is released under the terms of the AGPLv3 or higher
+///|/
 #ifdef _WIN32
 	#include <windows.h>
 	#include <boost/nowide/convert.hpp>
 #else
 	// any posix system
 	#include <pthread.h>
+	#ifdef __APPLE__
+		#include <pthread/qos.h>
+	#endif // __APPLE__
 #endif
 
 #include <atomic>
@@ -188,6 +195,32 @@ std::optional<std::string> get_current_thread_name()
 
 #endif // _WIN32
 
+// To be called at the start of the application to save the current thread ID as the main (UI) thread ID.
+static boost::thread::id g_main_thread_id;
+
+void save_main_thread_id()
+{
+	g_main_thread_id = boost::this_thread::get_id();
+}
+
+// Retrieve the cached main (UI) thread ID.
+boost::thread::id get_main_thread_id()
+{
+	return g_main_thread_id;
+}
+
+// Checks whether the main (UI) thread is active.
+bool is_main_thread_active()
+{
+	return get_main_thread_id() == boost::this_thread::get_id();
+}
+
+static thread_local ThreadData s_thread_data;
+ThreadData& thread_data()
+{
+	return s_thread_data;
+}
+
 // Spawn (n - 1) worker threads on Intel TBB thread pool and name them by an index and a system thread ID.
 // Also it sets locale of the worker threads to "C" for the G-code generator to produce "." as a decimal separator.
 void name_tbb_thread_pool_threads_set_locale()
@@ -203,7 +236,7 @@ void name_tbb_thread_pool_threads_set_locale()
 	const size_t nthreads_hw = tbb::this_task_arena::max_concurrency();
 	size_t       nthreads    = nthreads_hw;
 
-#ifdef SLIC3R_PROFILE
+#if 0
 	// Shiny profiler is not thread safe, thus disable parallelization.
 	disable_multi_threading();
 	nthreads = 1;
@@ -241,4 +274,27 @@ void name_tbb_thread_pool_threads_set_locale()
         });
 }
 
+void set_current_thread_qos()
+{
+#ifdef __APPLE__
+	// OSX specific: Set Quality of Service to "user initiated", so that the threads will be scheduled to high performance
+	// cores if available.
+	// With QOS_CLASS_USER_INITIATED the worker threads drop priority once slicer loses user focus.
+	pthread_set_qos_class_self_np(QOS_CLASS_USER_INTERACTIVE, 0);
+#endif // __APPLE__
+}
+
+void ThreadData::tbb_worker_thread_set_c_locales()
+{
+//    static std::atomic<int> cnt = 0;
+//    std::cout << "TBBLocalesSetter Entering " << cnt ++ << " ID " << std::this_thread::get_id() << "\n";
+    if (! m_tbb_worker_thread_c_locales_set) {
+        // Set locales of the worker thread to "C".
+        set_c_locales();
+        // OSX specific: Elevate QOS on Apple Silicon.
+        set_current_thread_qos();
+        m_tbb_worker_thread_c_locales_set = true;
+    }
+}
+
 }
