@@ -39,6 +39,7 @@ class MenuFactory
 public:
 	static std::vector<wxBitmapBundle*> get_volume_bitmaps();
 	static std::vector<wxBitmapBundle*> get_text_volume_bitmaps();
+	static std::vector<wxBitmapBundle*> get_svg_volume_bitmaps();
 
     MenuFactory();
     ~MenuFactory() = default;
@@ -56,6 +57,7 @@ public:
     wxMenu* sla_object_menu();
     wxMenu* part_menu();
     wxMenu* text_part_menu();
+    wxMenu* svg_part_menu();
     wxMenu* instance_menu();
     wxMenu* layer_menu();
     wxMenu* multi_selection_menu();
@@ -72,6 +74,7 @@ private:
     MenuWithSeparators m_object_menu;
     MenuWithSeparators m_part_menu;
     MenuWithSeparators m_text_part_menu;
+    MenuWithSeparators m_svg_part_menu;
     MenuWithSeparators m_sla_object_menu;
     MenuWithSeparators m_default_menu;
     MenuWithSeparators m_instance_menu;
@@ -87,10 +90,12 @@ private:
     void        append_mutable_part_menu_items(wxMenu* menu);
     void        create_part_menu();
     void        create_text_part_menu();
+    void        create_svg_part_menu();
     void        create_instance_menu();
 
     wxMenu*     append_submenu_add_generic(wxMenu* menu, ModelVolumeType type);
     void        append_menu_item_add_text(wxMenu* menu, ModelVolumeType type, bool is_submenu_item = true);
+    void        append_menu_item_add_svg(wxMenu *menu, ModelVolumeType type, bool is_submenu_item = true);    
     void        append_menu_items_add_volume(MenuType type);
     wxMenuItem* append_menu_item_layers_editing(wxMenu* menu);
     wxMenuItem* append_menu_item_settings(wxMenu* menu);
@@ -99,7 +104,7 @@ private:
     wxMenuItem* append_menu_item_printable(wxMenu* menu);
     void        append_menu_item_invalidate_cut_info(wxMenu *menu);
     void        append_menu_items_osx(wxMenu* menu);
-    wxMenuItem* append_menu_item_fix_through_netfabb(wxMenu* menu);
+    wxMenuItem* append_menu_item_fix_through_winsdk(wxMenu* menu);
     wxMenuItem* append_menu_item_simplify(wxMenu* menu);
     void        append_menu_item_export_stl(wxMenu* menu);
     void        append_menu_item_reload_from_disk(wxMenu* menu);
@@ -112,6 +117,7 @@ private:
 //    void        append_menu_item_merge_to_single_object(wxMenu *menu);
     void        append_menu_items_mirror(wxMenu *menu);
     void        append_menu_item_edit_text(wxMenu *menu);
+    void        append_menu_item_edit_svg(wxMenu *menu);
     void        append_menu_items_instance_manipulation(wxMenu *menu);
     void        update_menu_items_instance_manipulation(MenuType type);
     void        append_menu_items_split(wxMenu *menu);
